searchData={"items":[{"type":"module","title":"MagicAuth","doc":"MagicAuth is an authentication library for Phoenix that provides effortless configuration and flexibility for your project.\n\nKey Features:\n\n- **Passwordless Authentication**: Secure login process through one-time passwords sent via email\n- **Enhanced Security**: Protect your application from brute force attacks with built-in rate limiting and account lockout mechanisms\n- **Customizable Interface**: Fully customizable UI components to match your design\n- **Effortless Configuration**: Quick and simple integration with your Phoenix project\n- **Schema Agnostic**: Implement authentication without requiring a user schema - ideal for everything from MVPs to complex applications\n\nTo get started, see the installation documentation in `MagicAuth.MixProject`.","ref":"MagicAuth.html"},{"type":"function","title":"MagicAuth.children/0","doc":"Returns a list of child processes that should be supervised.\n\nIncludes token buckets needed for rate limiting:\n- OneTimePasswordRequestTokenBucket: Limits one-time password requests\n- LoginAttemptTokenBucket: Limits login attempts","ref":"MagicAuth.html#children/0"},{"type":"function","title":"Example - MagicAuth.children/0","doc":"In your application.ex (this configuration is automatically added by the `mix magic_auth.install` task):\n```elixir\nchildren = children ++ MagicAuth.children()\n```","ref":"MagicAuth.html#children/0-example"},{"type":"function","title":"MagicAuth.create_one_time_password/1","doc":"Creates and sends a one-time password for a given email.\n\nThe one-time password is stored in the database to allow users to log in from a device that\ndoesn't have access to the email where the code was sent. For example, the user\ncan receive the code on their phone and use it to log in on their computer.\n\nWhen called, this function creates a new one_time_password record and generates\na one-time password that will be used to authenticate it. The password is then passed\nto the configured callback module `one_time_password_requested/1` which should handle\nsending it to the user via email.\n\nOne-time password generation is rate limited using a token bucket system that allows a maximum of\n1 generation request per minute for each email address. This prevents abuse of the email delivery\nservice.","ref":"MagicAuth.html#create_one_time_password/1"},{"type":"function","title":"Parameters - MagicAuth.create_one_time_password/1","doc":"* `attrs` - A map containing `:email`","ref":"MagicAuth.html#create_one_time_password/1-parameters"},{"type":"function","title":"Returns - MagicAuth.create_one_time_password/1","doc":"* `{:ok, code, one_time_password}` - Returns the created one_time_password on success\n  * `{:error, changeset}` - Returns the changeset with errors if validation fails\n  * `{:error, failed_value}` - Returns the failed value if the transaction fails\n  * `{:error, :rate_limited, countdown}` - Returns the countdown if the rate limit is exceeded","ref":"MagicAuth.html#create_one_time_password/1-returns"},{"type":"function","title":"Examples - MagicAuth.create_one_time_password/1","doc":"iex> MagicAuth.create_one_time_password(%{\"email\" => \"user@example.com\"})\n    {:ok, code, %MagicAuth.OneTimePassword{}}\n\nThe one time password length can be configured in config/config.exs:\n\n```\nconfig :magic_auth,\n  one_time_password_length: 6 # default value\n```\n\nThis function:\n1. Removes any existing one_time_passwords for the provided email\n2. Creates a new one_time_password\n3. Generates a new random numeric password\n4. Encrypts the password using Bcrypt\n5. Stores the hash in the database\n6. Calls the configured callback module's `one_time_password_requested/1` function\n   which should handle sending the password to the user via email","ref":"MagicAuth.html#create_one_time_password/1-examples"},{"type":"function","title":"MagicAuth.fetch_magic_auth_session/2","doc":"Authenticates the user session by looking into the session and remember me token.\n\nThis function is designed to be used as a plug in your Phoenix router's pipeline.\nIt fetches the user's session from either the session storage or the remember me cookie,\nand assigns the session and user (if available) to the connection.\n\nWhen a user is successfully authenticated:\n- `assigns[:current_session]` will contain the session data\n- `assigns[:current_user]` will contain the user data only if:\n  1. A tuple `{:allow, user_id}` was returned from the `log_in_requested/1` callback during authentication\n  2. The user can be retrieved using the configured `get_user` function. This function\n     should be configured with `config :magic_auth, get_user: &MyApp.Accounts.get_user_by_id/1`. An error\n     will be raised if the tuple is returned from the callback and the user schema is not configured.\n\nIf the callback returned just `:allow` without a user_id, or if the user schema is not configured,\n`current_user` will be nil.","ref":"MagicAuth.html#fetch_magic_auth_session/2"},{"type":"function","title":"Examples - MagicAuth.fetch_magic_auth_session/2","doc":"# In your router.ex\n    pipeline :browser do\n      plug :accepts, [\"html\"]\n      plug :fetch_session\n      plug :fetch_live_flash\n      plug :put_root_layout, {MyAppWeb.LayoutView, :root}\n      plug :protect_from_forgery\n      plug :put_secure_browser_headers\n      plug :fetch_magic_auth_session\n    end","ref":"MagicAuth.html#fetch_magic_auth_session/2-examples"},{"type":"function","title":"MagicAuth.get_session_by_token/1","doc":"Gets the session with the given token.","ref":"MagicAuth.html#get_session_by_token/1"},{"type":"function","title":"MagicAuth.live_socket_id/1","doc":"","ref":"MagicAuth.html#live_socket_id/1"},{"type":"function","title":"MagicAuth.log_in/3","doc":"Logs the session in.\n\nIt renews the session ID and clears the whole session\nto avoid fixation attacks.\n\nLogin attempts are rate limited using a token bucket that allows a maximum of\n10 attempts every 10 minutes per email address.\n\nIt also sets a `:live_socket_id` key in the session,\nso LiveView sessions are identified and automatically\ndisconnected on log out.\n\nOn login success:\n  - Renews the session to prevent fixation attacks\n  - Sets the token in session and cookie (if remember me is enabled)\n  - Redirects to original page requested or to `/` (default route)\n\nOn error:\n  - If too many attempts: Redirects to `/sessions/log_in` with rate limit error message\n  - If invalid code: Redirects to `/sessions/password`\n  - If expired code: Redirects to `/sessions/password`\n  - If access denied: Redirects to `/sessions/log_in` with access denied error message","ref":"MagicAuth.html#log_in/3"},{"type":"function","title":"Denying access - MagicAuth.log_in/3","doc":"To deny access, implement the `log_in_requested/1` callback in your callback module\nreturning `:deny`. For example:\n\n```elixir\ndef log_in_requested(email) do\n  case Accounts.get_user_by_email(email) do\n    %User{active: false} -> :deny  # Denies access for inactive users\n    _ -> :allow\n  end\nend\n```\nFor more information on denying access, see the comments for the `log_in_requested/1` function\nin the generated MagicAuth module in your application's codebase.","ref":"MagicAuth.html#log_in/3-denying-access"},{"type":"function","title":"Parameters - MagicAuth.log_in/3","doc":"- `conn`: The Plug.Conn connection\n- `email`: String containing the user's email address\n- `code`: String containing the one-time password code","ref":"MagicAuth.html#log_in/3-parameters"},{"type":"function","title":"MagicAuth.log_out/1","doc":"Logs out and redirects to the log in page.\n\nIt clears all session data for safety.","ref":"MagicAuth.html#log_out/1"},{"type":"function","title":"MagicAuth.log_out_all/2","doc":"Logs out all sessions for a given user and redirects to the log in page.\n\nIt deletes all sessions associated with the user from the database and broadcasts\na disconnect message to all live views connected with those sessions.\n\nExcept for special cases, you should not use this function directly. If the user requests to log out of all sessions,\nyou should redirect to the logout page.","ref":"MagicAuth.html#log_out_all/2"},{"type":"function","title":"Parameters - MagicAuth.log_out_all/2","doc":"* `conn` - The Plug.Conn connection\n  * `opts` - Options for controlling the logout behavior:\n    * `:disconnect_self` - Boolean that determines whether to disconnect the current session socket (defaults to `true`)","ref":"MagicAuth.html#log_out_all/2-parameters"},{"type":"function","title":"Returns - MagicAuth.log_out_all/2","doc":"* `%Plug.Conn{}` - The updated connection with cleared session data and redirect to root path","ref":"MagicAuth.html#log_out_all/2-returns"},{"type":"function","title":"MagicAuth.log_out_all/3","doc":"Logs out all sessions for a given user and session.\n\nThis is a lower-level function that handles the actual session deletion and socket disconnection\nlogic. It's used by `log_out_all/2` which provides the connection-based interface.","ref":"MagicAuth.html#log_out_all/3"},{"type":"function","title":"Parameters - MagicAuth.log_out_all/3","doc":"* `user` - The user struct whose sessions should be terminated\n  * `session` - The current session struct\n  * `opts` - Options for controlling the logout behavior:\n    * `:disconnect_self` - Boolean that determines whether to disconnect the current session socket (defaults to `true`)","ref":"MagicAuth.html#log_out_all/3-parameters"},{"type":"function","title":"Returns - MagicAuth.log_out_all/3","doc":"* `:ok` - When the operation completes successfully","ref":"MagicAuth.html#log_out_all/3-returns"},{"type":"function","title":"MagicAuth.on_mount/4","doc":"Mount function for LiveViews that require authentication.\n\n  This function:\n  1. Mounts the user session on the socket\n  2. Assigns the current user to the socket if a tuple `{:allow, user_id}` was returned from the\n    `log_in_requested/1` callback during authentication\n  3. Continues the mount flow if user is authenticated\n  4. Halts and redirects to login page if user is not authenticated","ref":"MagicAuth.html#on_mount/4"},{"type":"function","title":"Examples of usage - MagicAuth.on_mount/4","doc":"In LiveView modules:\n```elixir\ndefmodule MyAppWeb.DashboardLive do\n  use MyAppWeb, :live_view\n\n  on_mount {MagicAuth, :require_authenticated}\n\n  def mount(_params, _session, socket) do\n    {:ok, socket}\n  end\nend\n```\n\nIn router.ex:\n```elixir\nlive_session :admin,\n  on_mount: [{MagicAuth, :require_authenticated}] do\n  live \"/dashboard\", DashboardLive\nend\n```","ref":"MagicAuth.html#on_mount/4-examples-of-usage"},{"type":"function","title":"MagicAuth.redirect_if_authenticated/2","doc":"Used for routes that require the user to not be authenticated.","ref":"MagicAuth.html#redirect_if_authenticated/2"},{"type":"function","title":"MagicAuth.require_authenticated/2","doc":"Plug function that verifies if the user is authenticated.\n\nIf the user is not authenticated:\n- Stores the current URL in the session for later redirect\n- Redirects to the login page (defaults to: /session/log_in)\n- Shows unauthorized error message\n- Halts request processing\n\nIf the user is authenticated:\n- Allows the request to continue normally with the session information\n- The current_session is available in conn.assigns[:current_session]","ref":"MagicAuth.html#require_authenticated/2"},{"type":"function","title":"Examples of usage - MagicAuth.require_authenticated/2","doc":"In router.ex routes:\n```elixir\nscope \"/\", MyAppWeb do\n  pipe_through [:browser, :require_authenticated]\n\n  get \"/dashboard\", DashboardController, :index\n  live \"/profile\", ProfileLive\nend\n```","ref":"MagicAuth.html#require_authenticated/2-examples-of-usage"},{"type":"function","title":"MagicAuth.verify_password/2","doc":"Verifies a one-time password for a given email.\n\nTakes an email and password as input and validates the one-time password.\n\nReturns:\n- `{:ok, one_time_password}` if the password is valid\n- `{:error, :invalid_code}` if the password is invalid or no password exists for email\n- `{:error, :code_expired}` if the password has expired\n\nThe function:\n1. Looks up the one-time password record for the given email\n2. Returns error if no password exists (with timing attack protection)\n3. Checks if password has expired based on configured expiration time\n4. Verifies the provided password matches the stored hash","ref":"MagicAuth.html#verify_password/2"},{"type":"module","title":"MagicAuth.Config","doc":"Configuration module for MagicAuth.\n\nThis module provides functions to access configuration values for the MagicAuth library.\nIt handles default values and required configurations.","ref":"MagicAuth.Config.html"},{"type":"module","title":"Configuration Options - MagicAuth.Config","doc":"- `:repo` - (required) The Ecto repository module used by your application.\n- `:router` - (required) Your application's router module.\n- `:callbacks` - (required) Module implementing MagicAuth callback functions.\n- `:endpoint` - (required) Your Phoenix application's endpoint module.\n- `:remember_me_cookie` - (required) Name of the cookie used for \"remember me\" functionality.\n- `:get_user` - required only if you want to use the `current_user` assign in your LiveView / Controllers.\n    If you are returning `{:allow, user_id}` from the `log_in_requested/1` callback, you need to\n    configure the `get_user` function to retrieve the user.\n- `:repo_opts` - (optional, default: `[magic_auth: true]`) Options passed to repository calls.\nCan be a keyword list or a function that returns options. Passing a function is useful to pass\noptions that are dependent on the current environment. For example, you can pass a function that\nreturns the current tenant prefix: `fn -> [prefix: MyApp.Repo.get_org_id()] end`.\n- `:one_time_password_length` - (optional, default: `6`) Length of generated one-time passwords.\n- `:one_time_password_expiration` - (optional, default: `10`) Expiration time in minutes for one-time passwords.\n- `:remember_me` - (optional, default: `true`) Whether to enable \"remember me\" functionality.\n- `:session_validity_in_days` - (optional, default: `60`) How long sessions remain valid.\n- `:enable_rate_limit` - (optional, default: `true`) Whether to enable rate limiting for authentication attempts.","ref":"MagicAuth.Config.html#module-configuration-options"},{"type":"module","title":"Configuration Example - MagicAuth.Config","doc":"In your `config/config.exs` file:\n\n```elixir\nconfig :magic_auth,\n  repo: MyApp.Repo,\n  router: MyAppWeb.Router,\n  callbacks: MyAppWeb.MagicAuthCallbacks,\n  endpoint: MyAppWeb.Endpoint,\n  remember_me_cookie: \"magic_auth_remember_me\",\n  get_user: &MyApp.Accounts.get_user/1,\n  # Optional configurations\n  one_time_password_length: 6,\n  one_time_password_expiration: 10,\n  remember_me: true,\n  session_validity_in_days: 60,\n  enable_rate_limit: true,\n  repo_opts: fn -> [magic_auth: true] end\n```","ref":"MagicAuth.Config.html#module-configuration-example"},{"type":"function","title":"MagicAuth.Config.callback_module/0","doc":"","ref":"MagicAuth.Config.html#callback_module/0"},{"type":"function","title":"MagicAuth.Config.endpoint/0","doc":"","ref":"MagicAuth.Config.html#endpoint/0"},{"type":"function","title":"MagicAuth.Config.get_user/0","doc":"","ref":"MagicAuth.Config.html#get_user/0"},{"type":"function","title":"MagicAuth.Config.one_time_password_expiration/0","doc":"","ref":"MagicAuth.Config.html#one_time_password_expiration/0"},{"type":"function","title":"MagicAuth.Config.one_time_password_length/0","doc":"","ref":"MagicAuth.Config.html#one_time_password_length/0"},{"type":"function","title":"MagicAuth.Config.rate_limit_enabled?/0","doc":"","ref":"MagicAuth.Config.html#rate_limit_enabled?/0"},{"type":"function","title":"MagicAuth.Config.remember_me/0","doc":"","ref":"MagicAuth.Config.html#remember_me/0"},{"type":"function","title":"MagicAuth.Config.remember_me_cookie/0","doc":"","ref":"MagicAuth.Config.html#remember_me_cookie/0"},{"type":"function","title":"MagicAuth.Config.repo_module/0","doc":"","ref":"MagicAuth.Config.html#repo_module/0"},{"type":"function","title":"MagicAuth.Config.repo_opts/0","doc":"","ref":"MagicAuth.Config.html#repo_opts/0"},{"type":"function","title":"MagicAuth.Config.router/0","doc":"","ref":"MagicAuth.Config.html#router/0"},{"type":"function","title":"MagicAuth.Config.session_validity_in_days/0","doc":"","ref":"MagicAuth.Config.html#session_validity_in_days/0"},{"type":"module","title":"MagicAuth.Multi","doc":"","ref":"MagicAuth.Multi.html"},{"type":"function","title":"MagicAuth.Multi.delete_all/3","doc":"","ref":"MagicAuth.Multi.html#delete_all/3"},{"type":"function","title":"MagicAuth.Multi.insert/3","doc":"","ref":"MagicAuth.Multi.html#insert/3"},{"type":"function","title":"MagicAuth.Multi.new/0","doc":"","ref":"MagicAuth.Multi.html#new/0"},{"type":"module","title":"MagicAuth.OneTimePassword","doc":"Ecto schema representing one-time passwords.\n\nThis module defines the data structure for one-time passwords, including the user's email and the\nencrypted password. It also provides functions to generate one-time passwords and to create changesets\nfor data validation.\n\nThe one-time password is generated based on the length configuration defined in\n`MagicAuth.Config.one_time_password_length()`. The password is composed only of numeric digits and is\ngenerated randomly using a strong random number generator.","ref":"MagicAuth.OneTimePassword.html"},{"type":"function","title":"MagicAuth.OneTimePassword.changeset/2","doc":"","ref":"MagicAuth.OneTimePassword.html#changeset/2"},{"type":"function","title":"MagicAuth.OneTimePassword.email_pattern/0","doc":"","ref":"MagicAuth.OneTimePassword.html#email_pattern/0"},{"type":"function","title":"MagicAuth.OneTimePassword.generate_code/0","doc":"","ref":"MagicAuth.OneTimePassword.html#generate_code/0"},{"type":"module","title":"MagicAuth.Repo","doc":"","ref":"MagicAuth.Repo.html"},{"type":"function","title":"MagicAuth.Repo.all/1","doc":"","ref":"MagicAuth.Repo.html#all/1"},{"type":"function","title":"MagicAuth.Repo.delete!/1","doc":"","ref":"MagicAuth.Repo.html#delete!/1"},{"type":"function","title":"MagicAuth.Repo.delete_all/1","doc":"","ref":"MagicAuth.Repo.html#delete_all/1"},{"type":"function","title":"MagicAuth.Repo.get/2","doc":"","ref":"MagicAuth.Repo.html#get/2"},{"type":"function","title":"MagicAuth.Repo.get_by/2","doc":"","ref":"MagicAuth.Repo.html#get_by/2"},{"type":"function","title":"MagicAuth.Repo.get_user/1","doc":"","ref":"MagicAuth.Repo.html#get_user/1"},{"type":"function","title":"MagicAuth.Repo.insert!/1","doc":"","ref":"MagicAuth.Repo.html#insert!/1"},{"type":"function","title":"MagicAuth.Repo.one/1","doc":"","ref":"MagicAuth.Repo.html#one/1"},{"type":"function","title":"MagicAuth.Repo.transaction/1","doc":"","ref":"MagicAuth.Repo.html#transaction/1"},{"type":"module","title":"MagicAuth.Router","doc":"Responsible for defining and managing MagicAuth authentication routes.\n\nThis module provides macros to configure authentication routes in Phoenix applications,\nallowing customization of login and password paths.","ref":"MagicAuth.Router.html"},{"type":"module","title":"Usage - MagicAuth.Router","doc":"To use it, add `use MagicAuth.Router` to your router module:\n\n```elixir\ndefmodule MyApp.Router do\n  use Phoenix.Router\n  use MagicAuth.Router\n\n  # Default configuration\n  magic_auth()\n\n  # Or with custom configuration\n  magic_auth(\"/auth\", log_in: \"/entrar\", password: \"/senha\", log_out: \"/sair\")\nend\n```\n\nFor more information about path customization, see the `magic_auth/2` macro.","ref":"MagicAuth.Router.html#module-usage"},{"type":"module","title":"Introspection Functions - MagicAuth.Router","doc":"The following functions are used internally to generate and manage authentication routes:\n\n- `__magic_auth__(:scope)` - Returns the configured base path\n- `__magic_auth__(:log_in, query)` - Returns the login path with optional query parameters\n- `__magic_auth__(:password, query)` - Returns the password path with optional query parameters\n- `__magic_auth__(:verify, query)` - Returns the verify path with optional query parameters\n- `__magic_auth__(:log_out, query)` - Returns the log out path with optional query parameters\n- `__magic_auth__(:signed_in, query)` - Returns the signed in path with optional query parameters\n\nThe `query` parameter is an optional map that allows adding query parameters to the generated URLs.","ref":"MagicAuth.Router.html#module-introspection-functions"},{"type":"module","title":"Example - MagicAuth.Router","doc":"```elixir\n  __magic_auth__(:log_in, %{foo: \"bar\", foo: \"bar\"})\n  # Returns: \"/sessions/login?foo=bar\n```","ref":"MagicAuth.Router.html#module-example"},{"type":"macro","title":"MagicAuth.Router.magic_auth/2","doc":"Macro to configure MagicAuth authentication routes.","ref":"MagicAuth.Router.html#magic_auth/2"},{"type":"macro","title":"Parameters - MagicAuth.Router.magic_auth/2","doc":"* `scope` - Base path for authentication routes. Default: \"/sessions\"\n  * `opts` - List of options to customize paths:\n    * `:log_in` - Path for login page. Default: \"/log_in\"\n    * `:password` - Path for password page. Default: \"/password\"\n    * `:verify` - Path for verify controller. Default: \"/verify\"\n    * `:log_out` - Path for log out controller. Default: \"/log_out\"\n    * `:signed_in` - Path to redirect user after a succesful login if no specific page is requested. Default: \"/\"","ref":"MagicAuth.Router.html#magic_auth/2-parameters"},{"type":"macro","title":"Default configuration - MagicAuth.Router.magic_auth/2","doc":"```elixir\nmagic_auth()\n```\n\nGenerates the following default routes:\n- /sessions/log_in\n- /sessions/password\n- /sessions/verify\n- /sessions/log_out","ref":"MagicAuth.Router.html#magic_auth/2-default-configuration"},{"type":"macro","title":"Custom configuration - MagicAuth.Router.magic_auth/2","doc":"```\nmagic_auth(\"/auth\", log_in: \"/entrar\", password: \"/senha\", verify: \"/verificar\", log_out: \"/sair\")\n```\n\nGenerates the following custom routes:\n- /auth/entrar\n- /auth/senha\n- /auth/verificar\n- /auth/sair","ref":"MagicAuth.Router.html#magic_auth/2-custom-configuration"},{"type":"macro","title":"Customizing the default sign in path - MagicAuth.Router.magic_auth/2","doc":"The default sign in path can be customized by passing the `:signed_in` parameter in the `magic_auth` macro call.\nThis allows developers to define a custom path for the sign in page after authentication.\n\nNote that Magic Auth will redirect users to their requested page. If no specific page is requested, users will\nbe redirected to the route configured in the `signed_in` option. For example, if a user accesses the application\nby visiting the `/sessions/log_in` page, they will be redirected to the `signed_in` route after the successful\nlog in.\n\nExample:\n```\nmagic_auth(\"/auth\", signed_in: \"/dashboard\")\n```\nThis will generate a custom sign in path to `/dashboard` instead of the default `/`.","ref":"MagicAuth.Router.html#magic_auth/2-customizing-the-default-sign-in-path"},{"type":"module","title":"MagicAuth.Session","doc":"Ecto schema representing user sessions.\n\nThis module defines the data structure for user sessions, including the user's email and the session token.\nIt also provides functions to generate session tokens and verify them.","ref":"MagicAuth.Session.html"},{"type":"function","title":"MagicAuth.Session.build_session/1","doc":"Generates a token that will be stored in a signed place,\nsuch as session or cookie. As they are signed, those\ntokens do not need to be hashed.\n\nThe reason why we store session tokens in the database, even\nthough Phoenix already provides a session cookie, is because\nPhoenix' default session cookies are not persisted, they are\nsimply signed and potentially encrypted. This means they are\nvalid indefinitely, unless you change the signing/encryption\nsalt.\n\nTherefore, storing them allows individual user\nsessions to be expired. The token system can also be extended\nto store additional data, such as the device used for logging in.\nYou could then use this information to display all valid sessions\nand devices in the UI and allow users to explicitly expire any\nsession they deem invalid.","ref":"MagicAuth.Session.html#build_session/1"},{"type":"function","title":"MagicAuth.Session.verify_session_token_query/2","doc":"Checks if the token is valid and returns its underlying lookup query.\n\nThe query returns the user found by the token, if any.\n\nThe token is valid if it matches the value in the database and it has\nnot expired (after @session_validity_in_days).","ref":"MagicAuth.Session.html#verify_session_token_query/2"},{"type":"module","title":"MagicAuth.TestHelpers","doc":"","ref":"MagicAuth.TestHelpers.html"},{"type":"function","title":"MagicAuth.TestHelpers.log_in_session/2","doc":"","ref":"MagicAuth.TestHelpers.html#log_in_session/2"},{"type":"task","title":"mix magic_auth.install","doc":"","ref":"Mix.Tasks.MagicAuth.Install.html"},{"type":"function","title":"Mix.Tasks.MagicAuth.Install.inject_config/0","doc":"","ref":"Mix.Tasks.MagicAuth.Install.html#inject_config/0"},{"type":"function","title":"Mix.Tasks.MagicAuth.Install.install_magic_auth_migration_file/0","doc":"","ref":"Mix.Tasks.MagicAuth.Install.html#install_magic_auth_migration_file/0"},{"type":"function","title":"Mix.Tasks.MagicAuth.Install.run/1","doc":"","ref":"Mix.Tasks.MagicAuth.Install.html#run/1"},{"type":"task","title":"mix magic_auth.setup_test_db","doc":"","ref":"Mix.Tasks.MagicAuth.SetupTestDb.html"},{"type":"function","title":"Mix.Tasks.MagicAuth.SetupTestDb.run/1","doc":"","ref":"Mix.Tasks.MagicAuth.SetupTestDb.html#run/1"},{"type":"extras","title":"🔒 Magic Auth","doc":"# 🔒 Magic Auth\n\nMagic Auth is an authentication library for Phoenix that provides effortless configuration and flexibility for your project.\n\n![Magic Auth in action](assets/magic_auth_in_action.gif)","ref":"getting_started.html"},{"type":"extras","title":"Key Features - 🔒 Magic Auth","doc":"- **Ship Faster** 🚀: No time wasted configuring password resets and recovery flows - just implement and ship your product.\n- **Passwordless Authentication** 📨: Secure login process through one-time passwords sent via email. One-time passwords are better than magic links because users can receive the code on one device (e.g., phone email) and enter it on another (e.g., desktop browser).\n- **Enhanced Security** 🔒: Protect your application from brute force attacks with built-in rate limiting and account lockout mechanisms.\n- **Customizable Interface** 🎨: Fully customizable UI components to match your design.\n- **Effortless Configuration and Comprehensive Documentation** 📚: Quick and simple integration with your Phoenix project, with detailed guides and references to assist you through every step of the integration process.\n- **Schema Agnostic** 👤: Implement authentication without requiring a user schema - ideal for everything from MVPs to complex applications.","ref":"getting_started.html#key-features"},{"type":"extras","title":"Installation - 🔒 Magic Auth","doc":"To install Magic Auth, add it to your dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:magic_auth, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"getting_started.html#installation"},{"type":"extras","title":"Getting started - 🔒 Magic Auth","doc":"Magic Auth simplifies the process of adding authentication to your Phoenix application. You'll have a fully functional authentication system integrated into your Phoenix application in less than 5 minutes. It's just 2 easy and quick steps!","ref":"getting_started.html#getting-started"},{"type":"extras","title":"Step 1 - Run generator - 🔒 Magic Auth","doc":"After adding Magic Auth to your dependencies, you can quickly set up authentication in your Phoenix application using the installation generator:\n\n```bash\nmix magic_auth.install\n```\n\nIf you are working within an umbrella project, navigate to your web application directory and run the generator:\n\n```bash\ncd apps/my_app_web/ && mix magic_auth.install\n```\n\nDon't forget to run the migrations to create the necessary tables for Magic Auth:\n\n```bash\nmix ecto.migrate\n```","ref":"getting_started.html#step-1-run-generator"},{"type":"extras","title":"Step 2 - Protect your routes - 🔒 Magic Auth","doc":"To protect your controllers and LiveViews with authentication, you need to configure the appropriate plugs and LiveView mounts. Edit the `lib/my_app_web/router.ex` file and modify it as shown in the example below:\n\n```elixir\ndefmodule MyAppWeb.Router do \n  # Additional router contents...\n\n  scope \"/\", MyAppWeb do\n    # Add MagicAuth.require_authenticated/2 plug to protect controllers\n    # and LiveView first mount (disconnected)\n    pipe_through [:browser, :require_authenticated]\n\n    get \"/protected_controller\", ProtectedController, :index\n\n    # Use MagicAuth.required_authenticated/4 to protect LiveView's socket connection\n    live_session :authenticated, on_mount: [{MagicAuth, :require_authenticated}] do\n      live \"/protected_live_view\", ProtectedLiveView\n    end\n  end\nend\n```\n\nFor more details, refer to `MagicAuth.require_authenticated/2` and `MagicAuth.on_mount/4`.","ref":"getting_started.html#step-2-protect-your-routes"},{"type":"extras","title":"Log out user or session","doc":"# Log out user or session\n\nMagic Auth provides several methods to log out users from their sessions. This guide explains how to implement logout functionality in your application, including logging out from a single session or all sessions simultaneously. You'll learn how to create logout links in both regular views and LiveViews, ensuring your users can securely end their sessions when needed.","ref":"log_out_user_or_session.html"},{"type":"extras","title":"Creating a log out link - Log out user or session","doc":"After setting up authentication, you must provide users with a way to log out. Magic Auth provides helper functions to generate the correct path for this action:\n\n```elixir\n<.link method=\"delete\" href={~p\"/sessions/log_out\"}>Logout</.link>\n```","ref":"log_out_user_or_session.html#creating-a-log-out-link"},{"type":"extras","title":"Logging out from all sessions - Log out user or session","doc":"Magic Auth allows users to log out from all active sessions across all devices. This is useful for security purposes when a user suspects unauthorized access to their account.\n\nTo create a link that logs out from all sessions, use the following code:\n\n```elixir\n<.link method=\"delete\" href={~p\"/sessions/log_out/all\"}>Logout</.link>\n```","ref":"log_out_user_or_session.html#logging-out-from-all-sessions"},{"type":"extras","title":"Logging out from a LiveView - Log out user or session","doc":"Since it's not possible to redirect to a route that accepts the DELETE method, Magic Auth alternatively creates a route to log out from all sessions using a GET method so it can be used within a LiveView event. For all other cases, you must use the DELETE method for logging out of sessions. Using the GET method to remove sessions indiscriminately, such as in `<.links>`, can cause incorrect behavior of your web application.\n\n```elixir\ndef handle_event(\"save\", _params, socket) do \n  {:noreply, redirect(socket, to: ~p\"/sessions/log_out/all/get\")}\nend\n```\n\nIf you want to log out from all sessions\n\nSee all generated routes in [Introspection Functions](/magic_auth/MagicAuth.Router.html#module-introspection-functions) section of the `MagicAuth.Router` documentation.","ref":"log_out_user_or_session.html#logging-out-from-a-liveview"},{"type":"extras","title":"Security considerations when email changes - Log out user or session","doc":"When a user changes their email address, it's important to invalidate all existing sessions to ensure security and verify the new email is valid. This practice helps:\n\n1. Confirm the user has access to the new email address by requiring them to log in again\n2. Prevent unauthorized access if the email change was malicious\n\nHere's how to implement this in your application:\n\n```\nMagicAuth.log_out_all(conn)\n```\n\nIf you are using a LiveView-based form to update a user's email, you won't have access to the `conn` struct. In this case, you can use `MagicAuth.log_out_all/3` instead.\n\n```\n# Don't disconnect self or you'll be unable to redirect the current session to the login screen\nMagicAuth.log_out_all(user, session, disconnect_self: false)\n\n# Don't forget to redirect the current LiveView to the login screen after log out all\nredirect(socket, to: destination)\n```","ref":"log_out_user_or_session.html#security-considerations-when-email-changes"},{"type":"extras","title":"Customization","doc":"# Customization\n\nThe generator will create a file at `lib/my_app_web/magic_auth.ex` (or at `apps/my_app_web/lib/my_app_web/magic_auth.ex` in an umbrella project). This file contains several callbacks that you can modify to match your application's needs. It is filled with comprehensive comments that guide you through customizing both the appearance and behavior of Magic Auth. For detailed instructions, please refer to the comments in the generated file. Below is a brief explanation of what can be customized:\n\n- The log in form appearance by modifying `log_in_form/1`.\n- The verification form appearance by modifying `verify_form/1`.\n- E-mail templates by modifying `one_time_password_requested/1`, `text_email_body/1`, and `html_email_body/1`.\n- Access control logic by modifying `log_in_requested/1`.\n- Error message translations by modifying `translate_error/1`.","ref":"customization.html"},{"type":"extras","title":"Multi-tenant applications","doc":"# Multi-tenant applications\n\nMagic Auth provides robust support for multi-tenancy, accommodating both [foreign keys](https://hexdocs.pm/ecto/multi-tenancy-with-foreign-keys.html) and [query prefixes](https://hexdocs.pm/ecto/multi-tenancy-with-query-prefixes.html) strategies. This flexibility allows developers to choose the approach that best fits their application's architecture and requirements.","ref":"multi_tenancy.html"},{"type":"extras","title":"Multi-tenancy with Foreign Keys - Multi-tenant applications","doc":"Assuming you followed the Ecto guide on multi-tenancy, your repo should look like this:\n\n```elixir\ndefmodule MyApp.Repo do\n  use Ecto.Repo, otp_app: :my_app\n\n  require Ecto.Query\n\n  @impl true\n  def prepare_query(_operation, query, opts) do\n    cond do\n      opts[:skip_org_id] || opts[:ecto_query] in [:schema_migration, :preload] ->\n        {query, opts}\n\n      org_id = opts[:org_id] ->\n        {Ecto.Query.where(query, org_id: ^org_id), opts}\n\n      true ->\n        raise \"expected org_id or skip_org_id to be set\"\n    end\n  end\nend\n```\n\nYou can modify the first condition of your `cond` clause to include `opts[:magic_auth]` like this:\n\n```elixir\nopts[:magic_auth] || opts[:skip_org_id] || opts[:ecto_query] in [:schema_migration, :preload] ->\n  {query, opts}\nend\n```\n\nAlternatively, if you prefer not to modify your `Repo`, you can configure Magic Auth to pass `skip_org_id: true` in\nall queries using the `:repo_opts` configuration:\n\n```elixir\n# config/config.ex\n\nconfig :magic_auth,\n  repo_opts: [skip_org_id: true]\n```","ref":"multi_tenancy.html#multi-tenancy-with-foreign-keys"},{"type":"extras","title":"Multi-tenancy with Query prefixes - Multi-tenant applications","doc":"You can dynamically set the database prefix by passing a function to the `:repo_opts` configuration:\n\n```elixir\n# config/config.exs\nconfig :magic_auth,\n  repo_opts: fn ->\n    [prefix: Process.get({MyApp.Repo, :org_id})]\n  end\n```","ref":"multi_tenancy.html#multi-tenancy-with-query-prefixes"},{"type":"extras","title":"Testing","doc":"# Testing \n\nTo test routes that require authentication in your Phoenix application, utilize the `MagicAuth.TestHelpers.log_in_session/2` function as demonstrated below:\n\n```\ndefmodule MyApp.MyModuleTest do\n  use MyApp.ConnCase, async: true\n  import MagicAuth.TestHelpers\n\n  setup do\n    conn = build_conn() |> Plug.Test.init_test_session(%{})\n    %{conn: conn}\n  end\n\n  test \"creates a session and puts token in session\", %{conn: conn} do\n    params = %{email: \"test@example.com\"}\n    conn = log_in_session(conn, params)\n    # Test assertions\n  end\nend\n```\n\nRemember to disable rate limiting in your test environment to prevent test failures.\n\n```\n# config/test.exs\nconfig :magic_auth,\n  enable_rate_limit: false\n```","ref":"testing.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [0.2.0]","ref":"changelog.html"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Making the messages more visible when the task `mix magic_auth.install` fails to inject code into the files\n  - If a tuple `{:allow, user_id}` is returned from the `log_in_requested/1` callback, Magic Auth will store the user `ID` in \n    `%Session{user_id: user_id}`. The user will then be automatically loaded by `fetch_magic_auth_session/2` and assigned \n    to `assigns` under the `:current_user` key.\n  - Support for multi tenancy with query prefixes and foreign keys.\n  - Add default logging of success and error messages in generated email-sending code\n  - Added test helpers to simplify testing. Refer to the [Testing Guide](guides/testing.md) for further details.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Removed `MagicAuth.delete_all_sessions_by_token()`. Use `MagicAuth.log_out()` instead.\n  - Removed `MagicAuth.delete_all_sessions_by_email()`. Use `MagicAuth.log_out_all()` instead.\n  - Magic Auth now redirects to the configured log in page instead of `/` after log out.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"How use automatic user loading feature on legacy project - Changelog","doc":"To support the automatic user loading feature, a few small changes must be made to your project before updating (only if you want to use this functionality):\n\n  1. Add `user_id` column on `magic_auth_sessions` table.\n  ```elixir\n  alter table(:magic_auth_sessions) do\n    add :user_id, :integer, null: true\n  end\n  ```\n\n  2. Add your user schema to the Magic Auth configuration on `config/config.exs`:\n  ```elixir\n  config :magic_auth,\n    get_user: %MyApp.Accounts.get_user_by_id/1\n  ```\n\n  DONE!\n\n## [0.1.1] - 2025-01-29","ref":"changelog.html#how-use-automatic-user-loading-feature-on-legacy-project"},{"type":"extras","title":"Fixed - Changelog","doc":"- One-time passwords are not being deleted after verified, allowing them to be reused multiple times until they expire.\n\n## [0.1.0] - 2025-01-22\n\nThis is the first release of `magic_auth`.","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Installation generator `mix magic_auth.install`\n- Email-based one-time password authentication\n- Customizable login and verification forms\n- Customizable email templates\n- Configurable access control logic\n- Customizable error message translations\n- Complete documentation with examples\n- Support for standard and umbrella Phoenix projects\n- Swoosh integration for email delivery\n- Comprehensive automated test suite","ref":"changelog.html#added"},{"type":"extras","title":"LICENSE","doc":"Copyright 2025 Gustavo Honorato\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this\nsoftware and associated documentation files (the “Software”), to deal in the Software\nwithout restriction, including without limitation the rights to use, copy, modify,\nmerge, publish, distribute, sublicense, and/or sell copies of the Software, and to\npermit persons to whom the Software is furnished to do so, subject to the following\nconditions:\n\nThe above copyright notice and this permission notice shall be included in all copies\nor substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,\nINCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A\nPARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT\nHOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF\nCONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE\nOR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}